#!/bin/bash
#

# build_mongodb.sh <product_directory> <e15|e17|e19|c2|c7> <debug|prof> [tar] [source]

usage()
{
   echo "USAGE: `basename ${0}` <product_directory> <e10|e14|e15> <debug|prof> [tar]"
}

# -------------------------------------------------------------------
# shared boilerplate
# -------------------------------------------------------------------

get_this_dir() 
{
    ( cd / ; /bin/pwd -P ) >/dev/null 2>&1
    if (( $? == 0 )); then
      pwd_P_arg="-P"
    fi
    reldir=`dirname ${0}`
    thisdir=`cd ${reldir} && /bin/pwd ${pwd_P_arg}`
}

get_ssibuildshims()
{
    # make sure we can use the setup alias
    if [ -z ${UPS_DIR} ]
    then
       echo "ERROR: please setup ups"
       exit 1
    fi
    source `${UPS_DIR}/bin/ups setup ${SETUP_UPS}`
    
    setup ssibuildshims ${ssibuildshims_version} -z ${product_dir}
}

get_arch()
{
    if [ ${OS1} = "Darwin" ]; then
      arch="osx"
    fi
    if [[ "${OS1}" == Linux ]]; then
      arch="linux"
      archver=rhel70
      if [[ "${OS}" == "c8-x86_64" ]] || [[ "${OS}" == "rh8-x86_64" ]]; then
	  archver=rhel80
      fi
      if [[ "${OS}" == "u16-x86_64" ]]; then
	  archver=ubuntu1604
      elif [[ "${OS}" == "u18-x86_64" ]]; then
	  archver=ubuntu1804
      elif [[ "${OS}" == "u20-x86_64" ]]; then
	  archver=ubuntu2004
      fi
    fi
}

set_compiler_flags()
{
  case ${cqual} in
    e1[0245])
      xcc=gcc
      local cc=gcc
      local cxx=g++
      local fc=gfortran
      local std=14
      ;;
    e1[789])
      xcc=gcc
      local cc=gcc
      local cxx=g++
      local fc=gfortran
      local std=17
      ;;
    e2[06])
      xcc=gcc
      local cc=gcc
      local cxx=g++
      local fc=gfortran
      local std=17
      ;;
    e13)
      xcc=clang
      local cc=clang
      local cxx=clang++
      local fc=gfortran
      local std=14
      ;;
    c[27])
      xcc=clang
      local cc=clang
      local cxx=clang++
      local fc=gfortran
      local std=17
      extra_opts+=(-DLLVM_ENABLE_LIBCXX=ON -DCLANG_DEFAULT_CXX_STDLIB=libc++)
      ;;
    c1[45])
      xcc=clang
      local cc=clang
      local cxx=clang++
      local fc=clang
      local std=17
      extra_opts+=(-DLLVM_ENABLE_LIBCXX=ON -DCLANG_DEFAULT_CXX_STDLIB=libc++)
      ;;
    *)
      echo "ERROR: unrecognized compiler qualifier ${cqual}"
      usage
      exit 1
  esac
  compiler_specs=(-DCMAKE_C_COMPILER=${cc} \
      -DCMAKE_CXX_COMPILER=${cxx} \
      -DCMAKE_Fortran_COMPILER=${fc} \
      -Dcxx${std}=ON \
      -DCMAKE_CXX_STANDARD=${std} \
      -DCMAKE_C_STANDARD=${std} \
      -Dfortran=ON
      )
}


# -------------------------------------------------------------------
# start processing
# -------------------------------------------------------------------

product_dir=${1}
basequal=${2}
cqual=$basequal
extraqual=${3}
maketar=${4}

if [ -z ${product_dir} ]
then
   echo "ERROR: please specify the local product directory"
   usage
   exit 1
fi

# -------------------------------------------------------------------
# package name and version
# -------------------------------------------------------------------

package=mongodb
cver=1.23.0
origpkgver=v6_0_2
pkgver=${origpkgver}a
ssibuildshims_version=v1_04_13
cppsourcetarname=mongo-cxx-driver-r3.7.0

get_this_dir

get_ssibuildshims

source define_basics --

get_arch

pkgdotver=`echo $origpkgver|sed -e 's/_/\./g'|sed -e 's/^v//g'`
if [[ "${arch}" == "linux" ]]; then
 pkgtarname=mongodb-linux-x86_64-${archver}-${pkgdotver}
elif [[ "${arch}" == "osx" ]];then
 pkgtarname=mongodb-osx-ssl-x86_64-${pkgdotver}
else
  echo "ERROR: I don't know how to build MongoDB UPS package on ${arch}!"
  exit 1
fi
csourcetarname=mongo-c-driver-${cver}


if [ "${maketar}" = "tar" ] && [ -d ${pkgdir}/lib ]
then
   ${SSIBUILDSHIMS_DIR}/bin/make_distribution_tarball ${product_dir} ${package} ${pkgver} ${fullqual}
   exit 0
fi

echo "building ${package} for ${OS}-${plat}-${qualdir} (flavor ${flvr})"

# -------------------------------------------------------------------
# package dependent stuff goes here
# -------------------------------------------------------------------

# declare now so we can setup
# fake ups declare
fakedb=${product_dir}/${package}/${pkgver}/fakedb
${SSIBUILDSHIMS_DIR}/bin/fake_declare_product ${product_dir} ${package} ${pkgver} ${fullqual}

# setup so we get the expected compiler
setup -B ${package} ${pkgver} -q ${fullqual} -z ${fakedb}:${PRODUCTS}
setup cmake v3_13_1
ups active

# Common configure args

common_config_args="--prefix=${pkgdir} --enable-shared"

if [ "${extraqual}" = "debug" ]; then
    cmake_build_type="Debug"
    extra_config_args="--enable-debug --enable-tracing"
elif [ "${extraqual}" = "prof" ]; then
    cmake_build_type="RelWithDebInfo"
    extra_config_args="--enable-debug-symbols=yes"
else
    cmake_build_type="Release"
fi

mkdir -p ${blddir}
if [ ! -d ${blddir} ]
then
   echo "ERROR: failed to create ${blddir}"
   exit 1
fi
mkdir -p ${sourcedir}
if [ ! -d ${sourcedir} ]
then
   echo "ERROR: failed to create ${sourcedir}"
   exit 1
fi

ncore=`${SSIBUILDSHIMS_DIR}/bin/ncores`
extra_command="-j ${ncore}"

bson_poly=-DBSONCXX_POLY_USE_STD=1
if [[ "${basequal}" == e1[045] ]]; then
	# Use MNMLSTC for c++14 build
	bson_poly=-DBSONCXX_POLY_USE_MNMLSTC=1
  compiler=gcc
  export CC=gcc
  export CXX=g++
  extra_command="stdver=c++14 ${extra_command}"
elif [[ "${basequal}" == e1[789] ]]; then
  compiler=gcc
  export CC=gcc
  export CXX=g++
  extra_command="stdver=c++17 ${extra_command}"
elif [[ "${basequal}" == e2[06] ]]; then
  compiler=gcc
  export CC=gcc
  export CXX=g++
  extra_command="stdver=c++17 ${extra_command}"
elif [[ "${basequal}" == c[27] ]]; then
  compiler=clang
  export CC=clang
  export CXX=clang++
  extra_command="stdver=c++17 ${extra_command}"
  extra_opts+=(-DLLVM_ENABLE_LIBCXX=ON -DCLANG_DEFAULT_CXX_STDLIB=libc++)
elif [[ "${basequal}" == c1[45] ]]; then
  compiler=clang
  export CC=clang
  export CXX=clang++
  extra_command="stdver=c++17 ${extra_command}"
  extra_opts+=(-DLLVM_ENABLE_LIBCXX=ON -DCLANG_DEFAULT_CXX_STDLIB=libc++)
#elif [[ "${basequal}" == "i1" ]]; then
#  compiler=icc
#  extra_command="stdver=c++11 ${extra_command}"
else
  ssi_die "ERROR: unrecognized qualifier $basequal."
fi

cd ${tardir}

if [[ "${arch}" == "linux" ]]; then
 wget --no-check-certificate https://github.com/mongodb/mongo-c-driver/releases/download/${cver}/${csourcetarname}.tar.gz
elif [[ "${arch}" == "osx" ]];then
 curl --location --insecure --fail -O https://github.com/mongodb/mongo-c-driver/releases/download/${cver}/${csourcetarname}.tar.gz
else
  echo "ERROR: I don't know how to build MongoDB UPS package on ${arch}!"
  exit 1
fi

set -x 

cd "${sourcedir}"
tar xf ${tardir}/${csourcetarname}.tar.gz || ssi_die "Unable to untar c source."

#tar xf ${tardir}/${cppsourcetarname}.tar.gz || ssi_die "Unable to untar cpp source."
#ELF 6/28/2019: Released version of mongo-cxx-driver has build issues. Using fixed commit for now
cppsourcetarname=mongo-cxx-driver
mkdir -p ${sourcedir}/${cppsourcetarname}
git clone https://github.com/mongodb/mongo-cxx-driver.git
cd ${cppsourcetarname}
git checkout 131fa1a67acd45c0eebcbdcfee42b212af8d2e80
cd ${sourcedir}


tar xf ${tardir}/${pkgtarname}.tgz || ssi_die "Unable to untar source."

for ff in ${patchdir}/*.patch;do
    if [ `grep -c cxx $ff` -gt 0 ];then
        cd $cppsourcetarname
    else
        cd $csourcetarname
    fi
    patch -b -p0 <$ff || ssi_die "ERROR: Could not apply patch ${ff}"
    cd $sourcedir
done

# Install Binaries
mkdir -p ${pkgdir}
mv ${pkgtarname}/* ${pkgdir}

# Configure & Make C Driver
cd ${csourcetarname}/build
#${sourcedir}/${csourcetarname}/configure --disable-automatic-init-and-cleanup --disable-shm-counters ${common_config_args} ${extra_config_args} || ssi_die "ERROR: configure failed."
PKG_CONFIG_PATH=${pkgdir}/lib/pkgconfig cmake "${compiler_specs[@]}" -DENABLE_AUTOMATIC_INIT_AND_CLEANUP=OFF -DENABLE_SHM_COUNTERS=OFF -DCMAKE_BUILD_TYPE=${cmake_build_type} -DCMAKE_INSTALL_PREFIX=${pkgdir} ${extra_opts:+"${extra_opts[@]}"} ..
make compiler=${compiler} ${extra_command} || ssi_die "ERROR: c-driver make failed."
make install || ssi_die "ERROR: c-driver make install failed."
cd "${sourcedir}"

# Configure & Make C++ Driver
cd ${cppsourcetarname}/build
set_compiler_flags
PKG_CONFIG_PATH=${pkgdir}/lib/pkgconfig cmake "${compiler_specs[@]}" ${bson_poly} -DCMAKE_BUILD_TYPE=${cmake_build_type} -DCMAKE_INSTALL_PREFIX=${pkgdir} ${extra_opts:+"${extra_opts[@]}"} ..
make compiler=${compiler} ${extra_command} || ssi_die "ERROR: cxx-driver make failed."
make install || ssi_die "ERROR: cxx-driver make install failed."
set +x

# Put includes in correct location
mkdir ${pkgdir}/../include
mv ${pkgdir}/include/bsoncxx/v*/bsoncxx ${pkgdir}/../include
mv ${pkgdir}/include/mongocxx/v*/mongocxx ${pkgdir}/../include
rm -rf ${pkgdir}/include/bsoncxx ${pkgdir}/include/mongocxx
mv ${pkgdir}/include/* ${pkgdir}/../include
rm -rf ${pkgdir}/include

# Install CMake Config Files
#cp -a ${patchdir}/mongodb ${pkgdir}/lib64
mkdir -p ${pkgdir}/lib64/mongodb/cmake
export MONGODB_DOT_VERSION=`echo $pkgver|sed -e 's/_/\./g'|sed -e 's/^v//g'`
export MONGODB_VERSION=$pkgver
envsubst \$MONGODB_DOT_VERSION,\$MONGODB_VERSION <${patchdir}/mongodb/cmake/mongodbConfigVersion.cmake >${pkgdir}/lib64/mongodb/cmake/mongodbConfigVersion.cmake
envsubst \$MONGODB_DOT_VERSION,\$MONGODB_VERSION <${patchdir}/mongodb/cmake/mongodbConfig.cmake >${pkgdir}/lib64/mongodb/cmake/mongodbConfig.cmake

cp ${patchdir}/*.sh ${pkgdir}/bin

# Really don't want the build files sticking around
rm -rf ${sourcedir}

# Clean up.
cd "${pkgdir}"

# libtool and pkg-config files not useful. Also backups.
rm -rvf lib/pkgconfig lib/*.la
find . -name '*~' -exec rm -vf \{\} \;

${SSIBUILDSHIMS_DIR}/bin/declare_product ${product_dir} ${package} ${pkgver} ${fullqual}

# -------------------------------------------------------------------
# common bottom stuff
# -------------------------------------------------------------------

# this should not complain
echo "Finished building ${package} ${pkgver}"
setup ${package} ${pkgver} -q ${fullqual} -z ${product_dir}:${PRODUCTS}
echo "mongodb is installed at ${MONGODB_FQ_DIR}"

# this must be last
if [ "${maketar}" = "tar" ] && [ -d ${pkgdir}/lib* ]
then
   ${SSIBUILDSHIMS_DIR}/bin/make_distribution_tarball ${product_dir} ${package} ${pkgver} ${fullqual}
fi

exit 0
